import 'dart:async';
import 'dart:convert';

import 'socket_client.dart';
import 'constants.dart';

class ComuniaClient {
  String userId;
  SocketClient _socketClient;

  Stream onNewMessage;
  Stream onOtherStartTyping;
  Stream onOtherStopTyping;
  Stream onOtherSeenMessage;
  Stream onServerProccessMessage;

  final StreamController _onNewMessageController = StreamController.broadcast();
  final StreamController _onOtherStartTypingController =
      StreamController.broadcast();
  final StreamController _onOtherStopTypingController =
      StreamController.broadcast();
  final StreamController _onOtherSeenMessageController =
      StreamController.broadcast();
  final StreamController _onServerProccessMessageController =
      StreamController.broadcast();

  ComuniaClient(String url, String apiKey, String userId) {
    this.userId = userId;
    _socketClient = SocketClient(url, apiKey, userId);

    setup();
  }

  void setup() async {
    _socketClient.onNewMessage.listen((data) {
      _processNewMessage(data);
    });

    onNewMessage = _onNewMessageController.stream;
    onOtherStartTyping = _onOtherStartTypingController.stream;
    onOtherStopTyping = _onOtherStopTypingController.stream;
    onOtherSeenMessage = _onOtherSeenMessageController.stream;
    onServerProccessMessage = _onServerProccessMessageController.stream;
  }

  Future<dynamic> init() async {
    return _socketClient.init();
  }

  void disconnect() async {
    _socketClient.socket.disconnect();
  }

  void sendNewMessage(toUserId, message) async {
    return _socketClient.sendNewMessage(toUserId, message);
  }

  void sendOnStartTyping(toUserId) async {
    return _socketClient.sendOnStartTyping(toUserId);
  }

  void sendOnStopTyping(toUserId) async {
    return _socketClient.sendOnStopTyping(toUserId);
  }

  void sendMessageViewed(toUserId, messageId) async {
    return _socketClient.sendMessageViewed(toUserId, messageId);
  }

  void _processNewMessage(data) async {
    _log(data);
    switch (data['type']) {
      case MT_NEW_MESSAGE:
        {
          var encodedMessage = data['data'].replaceAll(RegExp(r'(?:[\t ]*(?:\r?\n|\r))+'), '');
          var text = utf8.decode(base64.decode(encodedMessage));

          var message = {
            'messageId': data['message_id'],
            'text': text,
            'from': data['sender'],
            'receivedTime': data['message_received_time']
          };

          _onNewMessageController.add(message);
        }
        break;

      case MT_START_TYPING:
        {
          _onOtherStartTypingController.add(data['sender']);
        }
        break;

      case MT_STOP_TYPING:
        {
          _onOtherStopTypingController.add(data['sender']);
        }
        break;

      case MT_SEEN:
        {
          var message = {
            'from': data['sender'],
            'messageId': data['message_id']
          };

          _onOtherSeenMessageController.add(message);
        }
        break;

      case MT_PROCESSED_BY_SERVER:
        {
          var message = {'messageId': data['message_id'], 'time': data['time']};

          _onServerProccessMessageController.add(message);
        }
        break;

      default:
        {
          print("Unknown message type: $data['type'], message: $data['data']");
        }
        break;
    }
  }

  void _log(data) {
    print('comunia > $data');
  }
}
