import 'package:cpass_flutter_sdk/cpass_flutter_sdk.dart';
import 'package:test/test.dart';
import 'config.dart';

void main() {
  group('Socket Basic Tests', () {
    setUp(() {});

    test('Socket client initialize succeffully', () async {
      var client = ComuniaClient(URL, API_KEY, 'dart-user1');
      var response = await client.init();
      client.disconnect();

      expect(response['message'], equals('success'));
      expect(response['response_code'], equals(200));
      return;
    });
  });

  group('Single Client Tests', () {
    test('Send message', () {
      var client = ComuniaClient(URL, API_KEY, 'dart-user2');
      client.init().then((value) {
        expect(value, equals(true));

        var receiverId = 'dart-user3';
        var message = 'Hello World';
        client.sendNewMessage(receiverId, message);
      });
    });
  });

  group('Multiple Client Test', () {
    test('Send and reply to a message', () {
      var testMessage = 'hello comunia';

      var user1 = 'dart-user2';
      var client1 = ComuniaClient(URL, API_KEY, user1);
      client1.init().then((isClient1Connected) {
        expect(isClient1Connected, equals(true));

        var user2 = 'dart-user3';
        var client2 = ComuniaClient(URL, API_KEY, user2);
        client2.init().then((isClient2Connected) {
          expect(isClient2Connected, equals(true));

          client2.onOtherStartTyping.listen(expectAsync1((user) {
            expect(user, equals(user1));
          }));

          client2.onNewMessage.listen(expectAsync1((msg) {
            expect(msg['from'], equals(user1));
            expect(msg['text'], equals(testMessage));
          }));

          client1.sendOnStartTyping(user2);
          client1.sendNewMessage(user2, testMessage);
        });
      });
    });
  });
}
